from sqlalchemy.orm import Session
import user_model 
import user_schema
import bcrypt
import os
from typing import Dict


def get_user(db: Session, user_id: int):
    return db.query(user_model.User).filter(user_model.User.id == user_id).first()


def get_user_by_email(db: Session, email: str):
    return db.query(user_model.User).filter(user_model.User.email == email).first()

def get_user_by_github_username(db: Session, username: str):
    return db.query(user_model.User).filter(user_model.User.github_username == username).first()

def get_users(db: Session, skip: int = 0, limit: int = 100):
    return db.query(user_model.User).offset(skip).limit(limit).all()

def reset_password(db: Session, email: str):
    user = db.query(user_model.User).filter(user_model.User.email == email).first()
    if not user:
        return None
    salt = bcrypt.gensalt()
    hashed = bcrypt.hashpw('changeme'.encode(), salt)
    user.hashed_password = hashed
    db.commit()
    return user


def create_admin(db: Session):
    print("Checking for admin user")
    admin = db.query(user_model.User).filter(user_model.User.email == "admin").first()
    if admin:
        print("Admin user already created")
        return
    print("Creating admin user")
    passwd = os.environ["SECRET_adminpass"]
    salt = bcrypt.gensalt()
    hashed = bcrypt.hashpw(passwd.encode("utf-8"), salt)
    admin = user_model.User(email="admin", roles="admin,dev", hashed_password=hashed.decode("utf-8"))
    db.add(admin)
    db.commit()
    db.refresh(admin)

def login(db: Session, params: user_schema.LoginParams):
    user = get_user_by_email(db, params.email)
    if not user:
        print("User {} doesn't exist".format(user))
        return None
    if bcrypt.checkpw(params.password.encode("utf-8"), user.hashed_password.encode("utf-8")):
        return user
    print("Password incorrect for user " + str(user.email))
    return None

def create_user(db: Session, user: user_schema.UserCreate):
    salt = bcrypt.gensalt()
    hashed = bcrypt.hashpw(user.password.encode("utf-8"), salt)
    db_user = user_model.User(email=user.email, github_org=user.github_org, github_username=user.github_username, roles=user.roles, hashed_password=hashed.decode("utf-8"))
    db.add(db_user)
    db.commit()
    db.refresh(db_user)
    return db_user
