set -e

HOST=api
AUTH_HOST="auth-api:8081"
PATH_LIST=("/api/environments" "/api/environments?app=f8&name=$MY_ENV&expected_count=1" "/api/init?repo=https://github.com/aramse/httpsql.git")

for i in $(seq 1 5); do  # retries in case intermittent auth/db issues
    TOKEN=$(curl -X POST $AUTH_HOST/auth/login -d "{\"email\": \"admin\", \"password\": \"$SECRET_adminpass\"}" | jq -r '.access_token' | tr -d '\n') && break
    sleep 15
done
AUTH="Authorization: Bearer $TOKEN"

set -x

for path in "${PATH_LIST[@]}"; do
  curl  --fail -v $HOST$path -H "$AUTH" > /dev/null || exit 1
done

# test schema validation
curl -X POST --fail $HOST/api/validate -d '{"general": {"name": "foo"}, "components": [{"name": "bar"}]}' -H "$AUTH" | grep 'failed'
curl -X POST --fail $HOST/api/validate -d '{"general": {"name": "foo"}, "components": [{"name": "bar", "type": "stateless", "containers": [], "expose": []}]}'  -H "$AUTH"  | grep 'successful'

# test env-type resolution
RESOLVE_URL="$HOST/api/resolve?env_type=local&include_globals=true&fmt=json"
expected_cfg='{"errors": [], "result": {"general": {"name": "foo"}, "components": [{"type": "stateless", "name": "foo", "containers": [{"dir": "foo", "imagePullPolicy": "Never"}], "expose": [{"servicePort": 80, "containerPort": 80, "external": true}]}]}}'

resolved_cfg=$(curl  -X POST --fail "$RESOLVE_URL" -d '{"general": {"name": "foo"}, "components": [{"type": "stateless", "name": "foo", "containers": [{"dir": "foo"}], "expose": [{"servicePort": 80, "containerPort": 80}]}]}' -H "$AUTH"  )
[ "$resolved_cfg" == "$expected_cfg" ] || exit 1

resolved_cfg=$(curl -X POST --fail "$RESOLVE_URL" -d '{"general": {"name": "foo"}, "components": [{"type": "stateless", "name": "foo", "containers": [{"dir": "foo", "imagePullPolicy": "Always"}], "expose": [{"servicePort": 80, "containerPort": 80}]}]}'  -H "$AUTH" )
[ "$resolved_cfg" == "$expected_cfg" ] || exit 1

input_cfg='{"general": {"name": "foo"}, "components": [{"type": "stateless", "name": "foo", "containers": [], "expose": []}, {"type": "stateless", "name": "bar", "containers": [], "expose": []}]}'
expected_cfg="{\"errors\": [], \"result\": $input_cfg}"

resolved_cfg=$(curl -X POST --fail "$RESOLVE_URL"  -d "$input_cfg" -H "$AUTH" )
[ "$resolved_cfg" == "$expected_cfg" ] || exit 1
