import { MediaMatcher } from '@angular/cdk/layout';
import { ChangeDetectorRef, Component, OnDestroy, ViewChild } from '@angular/core';
import { InfoDialogComponent } from '../info-dialog/info-dialog.component';
import { MatDialog } from '@angular/material/dialog';
import { GeneralService } from '../../app/services/general.service';
import { DocsService } from '../../app/services/docs.service';
import { Title } from '@angular/platform-browser';
import { AfterViewInit } from '@angular/core';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { SelectionModel } from '@angular/cdk/collections';
import { User } from '../models/user';
import { AuthService } from '../services/auth.service';
import { UserCreateComponent } from '../user-create/user-create.component';
import { ConfirmDialogComponent } from '../confirm-dialog/confirm-dialog.component';
import { MatSnackBar} from '@angular/material/snack-bar';


@Component({
  selector: 'app-admin',
  templateUrl: './admin.component.html',
  styleUrls: ['./admin.component.css']
})
export class AdminComponent implements OnDestroy, AfterViewInit {

  displayedColumns: string[] = ['select', 'email', 'roles', 'github_username', 'github_org'];
  dataSource = new MatTableDataSource<User>();
  selection = new SelectionModel<User>(true, []);

  @ViewChild(MatPaginator) paginator: MatPaginator;

  ngAfterViewInit() {
    this.dataSource.paginator = this.paginator;
  }

  jQuery;
  mobileQuery: MediaQueryList;
  private _mobileQueryListener: () => void;


  infoDialog: string =
    "<p>" +
    "Admin Panel" +
    "</p>";

  constructor(changeDetectorRef: ChangeDetectorRef,
    media: MediaMatcher,
    public dialog: MatDialog,
    private docsService: DocsService,
    private titleService: Title,
    private authService: AuthService,
    private snack: MatSnackBar) {
    titleService.setTitle('Admin Panel');
    this.mobileQuery = media.matchMedia('(max-width: 600px)');
    this._mobileQueryListener = () => changeDetectorRef.detectChanges();
    this.mobileQuery.addListener(this._mobileQueryListener);
  }

  ngOnDestroy(): void {
    this.mobileQuery.removeListener(this._mobileQueryListener);

  }

  createUser() {
    const dialogRef = this.dialog.open(UserCreateComponent,
      {
        width: '60%',
        height: '75%'
      }).afterClosed().subscribe(result => {
        this.refreshData();
      });
  }

  editUser() {
    if (this.selection.selected.length > 1 ) {
      this.snack.open("Please Select Only One User to Edit", "Dismiss");
    }
    const dialogRef = this.dialog.open(UserCreateComponent,
      {
        width: '60%',
        height: '75%',
        data: {
          user:  this.selection.selected[0],
        }
      }).afterClosed().subscribe(result => {
        this.refreshData();
      });
  }

  deleteUser() {
    const dialogRef = this.dialog.open(ConfirmDialogComponent,
      {
        width: '60%',
        height: '50%',
        data: {
          title: 'Confirm User Delete',
          message: `Are you sure you want to delete user(s)`
        }
      });
    dialogRef.afterClosed().subscribe(
      (result: boolean) => {
        if(result) {
          this.selection.selected.forEach(user => {
            this.authService.deleteUser(user.email).subscribe(()=> this.refreshData());
          });
        }
      }
    )
   
  }

  refreshData(){
    this.authService.getUsers().subscribe((data: User[]) => {
      this.dataSource.data = data;
    });
  }

  resetPassword() {
    const dialogRef = this.dialog.open(ConfirmDialogComponent,
      {
        width: '60%',
        height: '50%',
        data: {
          title: 'Confirm Reset Password',
          message: `Are you sure you want to reset User's Password`
        }
      });
  }

  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.data.length;
    return numSelected === numRows;
  }

  masterToggle() {
    this.isAllSelected() ?
      this.selection.clear() :
      this.dataSource.data.forEach(row => this.selection.select(row));
  }

  openInfoDialog(): void {
    const dialogRef = this.dialog.open(InfoDialogComponent,
      {
        width: '40%',
        height: '40%',
        data: {
          type: "Admin",
          text: this.infoDialog
        }
      }
    );
  }
  ngOnInit(): void {
    this.refreshData();
  }
}