import { BrowserModule } from '@angular/platform-browser';
import { NgModule } from '@angular/core';
import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { HttpClientModule } from '@angular/common/http';
import { FormsModule } from '@angular/forms';

import {MatFormFieldModule} from '@angular/material/form-field';
import {MatSelectModule} from '@angular/material/select';

import { MatInputModule } from '@angular/material/input';
import { MatButtonModule }  from '@angular/material/button';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatMenuModule } from '@angular/material/menu';

import { MatToolbarModule } from '@angular/material/toolbar';
import { MatSidenavModule } from '@angular/material/sidenav';
import { MatIconModule } from '@angular/material/icon';
import { MatListModule } from '@angular/material/list';
import { MatGridListModule}  from '@angular/material/grid-list';
import { MatDialogModule } from '@angular/material/dialog';
import { MatCardModule } from '@angular/material/card';
import { MatTableModule } from '@angular/material/table';
import {MatCheckboxModule} from '@angular/material/checkbox';
import { MatSnackBarModule } from '@angular/material/snack-bar';

import { LoginComponent } from './login/login.component';
import { HomeComponent } from './home/home.component';
import { ApplicationsComponent } from './applications/applications.component';
import { EnvironmentsComponent } from './environments/environments.component';
import { AppSelectorComponent } from './applications/app-selector/app-selector.component'; 
import { AppDialogComponent } from './applications/app-dialog/app-dialog.component';
import { SidenavComponent } from './sidenav/sidenav.component';
import { AddAppDialogComponent } from './applications/add-app-dialog/add-app-dialog.component';
import { CompTemplateComponent } from './comp-templates/comp-template.component';

import { ReactiveFormsModule } from '@angular/forms';
import { InfoDialogComponent } from './info-dialog/info-dialog.component';
import { Globals} from './global';
import { ConfigBuilderComponent } from './config-builder/config-builder.component';
import { InitDialogComponent } from './config-builder/init-dialog/init-dialog.component';
import { EnvGraphComponent } from './environments/env-graph/env-graph.component';
import { EnvLogComponent } from './environments/env-log/env-log.component';
import { DocsComponent } from './docs/docs.component';
import { CommonModule } from '@angular/common';

import { AuthService } from './services/auth.service';
import {HTTP_INTERCEPTORS } from '@angular/common/http';
import { TokenInterceptorService } from './services/token-interceptor.service';
import { environment } from '../environments/environment';
import { UrlInterceptorService } from './services/url-interceptor.service';
import { AuthGuardService } from './services/authguard.service';
import { LoginErrorComponent } from './login-error/login-error.component';
import { CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { CommitDialogComponent } from './commit-dialog/commit-dialog.component';
import { ConfirmDialogComponent } from './confirm-dialog/confirm-dialog.component';
import { AdminComponent } from './admin/admin.component';
import { UserCreateComponent } from './user-create/user-create.component';



@NgModule({
  schemas: [CUSTOM_ELEMENTS_SCHEMA],
  declarations: [
    AppComponent,
    HomeComponent,
    ApplicationsComponent,
    EnvironmentsComponent,
    AppSelectorComponent,
    AppDialogComponent,
    SidenavComponent,
    AddAppDialogComponent,
    InfoDialogComponent,
    ConfigBuilderComponent,
    InitDialogComponent,
    EnvGraphComponent,
    EnvLogComponent,
    DocsComponent,
    LoginComponent,
    LoginErrorComponent,
    CommitDialogComponent,
    ConfirmDialogComponent,
    AdminComponent,
    UserCreateComponent,
    CompTemplateComponent
  ],
  imports: [
    BrowserModule,
    AppRoutingModule,
    BrowserAnimationsModule,
    HttpClientModule,
    FormsModule,
    MatToolbarModule,
    MatSidenavModule,
    MatIconModule,
    MatListModule,
    MatGridListModule,
    MatDialogModule,
    MatCardModule,
    MatTableModule,
    MatFormFieldModule,
    MatSelectModule,
    ReactiveFormsModule,
    CommonModule,
    MatToolbarModule,
    MatIconModule,
    MatTableModule,
    MatInputModule,
    MatButtonModule,
    MatProgressSpinnerModule,
    MatMenuModule,
    MatCheckboxModule,
    MatSnackBarModule
  ],
  providers: [
    AuthGuardService,
    AuthService,
    Globals,
    {
      provide: HTTP_INTERCEPTORS,
      useClass: TokenInterceptorService,
      multi: true
    },
    {
      provide: HTTP_INTERCEPTORS,
      useClass: UrlInterceptorService,
      multi: true
    },
    { provide: "API_URL", useValue: environment.apiUrl},
    { provide: "AUTH_API_URL", useValue: environment.authApiUrl }
  ],
  bootstrap: [AppComponent]
})
export class AppModule { }
