import { Component, OnInit } from '@angular/core';
import { FormControl, Validators, FormGroup } from '@angular/forms';
import { ApplicationsService } from '../../services/applications.service';
import { Globals } from 'src/app/global';
import { MatDialogRef } from '@angular/material/dialog';

@Component({
  selector: 'app-add-app-dialog',
  templateUrl: './add-app-dialog.component.html',
  styleUrls: ['./add-app-dialog.component.css']
})
export class AddAppDialogComponent implements OnInit {
  creationSuccessful:boolean;
  creationErrorMsg:string;
  loading = 0;

  
  addAppForm = new FormGroup({
    name: new FormControl('',[Validators.required, Validators.pattern('[a-z][a-z0-9-]*[a-z0-9]')]),
    repo: new FormControl('',[Validators.required, Validators.pattern('http(s?)\:\/\/[0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*(:(0-9)*)*(\/?)([a-zA-Z0-9\-\.\?\,\'\/\\\+&amp;%\$#_]*)?\.git')])
  })
  onboardedName:string = "";
  onboardedGitRepo:string;


  get name(): any{
    return this.addAppForm.get('name');
  }
  get repo(): any{
    return this.addAppForm.get('repo');
  }
  constructor(private applicationsService: ApplicationsService,
     private global: Globals,
     public dialogRef:MatDialogRef<AddAppDialogComponent>) {
   }

  ngOnInit(): void {
  }

  addApp(){

    this.loading++;
    this.dialogRef.disableClose=true;
    this.addAppForm.get('name').disable();
    this.addAppForm.get('repo').disable();
    this.applicationsService.createApp(this.addAppForm.get('name').value, this.addAppForm.get('repo').value).
      subscribe(
        (response) => {
          if (response['successful']){
            this.onboardedName = this.addAppForm.get('name').value;
            this.onboardedGitRepo = this.addAppForm.get('repo').value;
            this.creationSuccessful = true;
            delete this.creationErrorMsg;
          } else{
            this.creationErrorMsg = response['errors'];
            this.creationSuccessful = false;
            this.addAppForm.get('name').enable();
            this.addAppForm.get('repo').enable();
          }
          this.loading--;
          this.dialogRef.disableClose=false;
        },
        (error)=>{
          this.creationErrorMsg = this.global.ORCH_CONNECTION_ERROR;
          this.creationSuccessful = false;
          this.loading--;
          this.dialogRef.disableClose=false;
          this.addAppForm.get('name').enable();
          this.addAppForm.get('repo').enable();
        }
      )
  }

}
