import {MediaMatcher} from '@angular/cdk/layout';
import {ChangeDetectorRef, Component, OnDestroy} from '@angular/core';
import {ApplicationsService} from '../services/applications.service';
import {GeneralService} from '../services/general.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Subscription, fromEvent, Subject} from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import { AppDialogComponent } from './app-dialog/app-dialog.component';
import { AddAppDialogComponent } from './add-app-dialog/add-app-dialog.component';
import { InfoDialogComponent } from '../info-dialog/info-dialog.component';
import { MatDialog } from '@angular/material/dialog';
import { Globals } from '../global'
import { takeUntil } from 'rxjs/operators';
import { Title } from '@angular/platform-browser';



@Component({
  selector: 'applications',
  templateUrl: './applications.component.html',
  styleUrls: ['./applications.component.css']
})
export class ApplicationsComponent implements OnDestroy {
  protected ngUnsubscribe: Subject<void> = new Subject<void>();

  mobileQuery: MediaQueryList;

  infoDialog:string = 
  "<p>"+
      "This page shows all applications deployed with f8. Click on any application to view its details, including:"+
    "<ul>"+
      "<li>Its architecture (based on a chosen environment) as well as links to externally accessible components.</li>"+
      "<li>All of its deployed environments, with links to the Environment page of each.</li>"+
      "<li>Links to related infrastructure like builds and monitoring.</li>"+
    "</ul>"+
      "You can also onboard a new application to f8 by clicking on the plus icon."+
  "</p>";

  errors={};

  appSelectors: String[]=[];
  env=[];
  apps=[];
  appsDisplayed=[];
  loading=0;

  routeQueryParams$: Subscription;
  queryParamApp;

  searchWord:string='';
  searchBox;
  keyup$;



  //TODO: dig into more------------------------------------------------
  private _mobileQueryListener: () => void;
  constructor(private generalService: GeneralService, 
    private applicationsService: ApplicationsService, 
    changeDetectorRef: ChangeDetectorRef, 
    media: MediaMatcher,
    public dialog: MatDialog,
    private route: ActivatedRoute,
    private router:Router,
    private global: Globals,
    private titleService: Title) {

    titleService.setTitle('Applications');
    this.mobileQuery = media.matchMedia('(max-width: 600px)');
    this._mobileQueryListener = () => changeDetectorRef.detectChanges();
    this.mobileQuery.addListener(this._mobileQueryListener);
    this.routeQueryParams$ = route.queryParams.subscribe(params => {
      if (params['app']) {
        this.queryParamApp=params['app'];
      }
    });
  }

  ngOnDestroy(): void {
    this.mobileQuery.removeListener(this._mobileQueryListener);
    this.routeQueryParams$.unsubscribe();
    
    if (this.keyup$ && this.keyup$.unsubscribe === 'function'){
      this.keyup$.unsubscribe();
    }

    // This aborts all HTTP requests.
    this.ngUnsubscribe.next();
    // This completes the subject properlly.
    this.ngUnsubscribe.complete();
    
  }
  //--------------------------------------------------------------------

  ngOnInit() {

    this.updateEnvs();
    if (this.queryParamApp){
      this.openQueryAppDialog(this.queryParamApp);
    }
  } 


  updateEnvs(){
    this.loading++;
    this.applicationsService.getEnvs()
    .pipe( takeUntil(this.ngUnsubscribe) )
    .subscribe(
      (data)=>{
        this.env=data['results'];
     
        this.apps=this.generalService.getApps(this.env);
        this.loading--;

        //set up search box
        this.searchBox = document.getElementById('app-search');
        this.keyup$ = fromEvent(this.searchBox, 'keyup')
        this.keyup$.pipe(
          debounceTime(500)
        ).subscribe(() => this.doSearch());
        this.doSearch();
        if (this.errors['general']){
          delete this.errors['general'];
        }
      },
      (error) => {
        this.errors['general']=this.global.ORCH_CONNECTION_ERROR;
        this.loading--;
      }
    )
  }

  // removeQueryParameters() {
  //   this.router.navigate([], {
  //     queryParams: {
  //       app: null,
  //     },
  //     queryParamsHandling: 'merge'
  //   })
  // }

  openQueryAppDialog(qParam:string): void{
    const dialogRef = this.dialog.open(AppDialogComponent, 
      {
        width: '90%',
        height: '90%',
        data: {
          name:qParam,
        },
      }
    );

    dialogRef.afterClosed().subscribe(() => {
      //this.removeQueryParameters();
      this.generalService.alterQueryParameters({
        app: null,
      })
    }
  )
  }


  openAddAppDialog(): void{
    const dialogRef = this.dialog.open(AddAppDialogComponent, 
      {
      width: '80%',
      height: '90%',
      }
    );
  }

  doSearch(){
    if (this.searchWord!==''){
      let searchedArray=[];
      for (let app of this.apps){
        if (app['name'].indexOf(this.searchWord)>=0){
          searchedArray.push(app);
        }
      }
      this.appsDisplayed = searchedArray;
    } else {
      this.appsDisplayed = this.apps;
    }
  }

  refresh(){
    if (this.loading===0){
      this.updateEnvs();
    }
  }

  openInfoDialog(): void{
    const dialogRef = this.dialog.open(InfoDialogComponent, 
      {
      width: '60%',
      height: '45%',
      data: {
        type:"applications",
        text:this.infoDialog
      }
      }
    );
  }
}
