import {MediaMatcher} from '@angular/cdk/layout';
import {ChangeDetectorRef, Component, OnDestroy} from '@angular/core';
import { InfoDialogComponent } from '../info-dialog/info-dialog.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { ActivatedRoute, Router } from '@angular/router';
import { Subscription, fromEvent, Subject} from 'rxjs';
import { CompTemplatesService } from '../../app/services/comp-templates.service';

/** @title Responsive sidenav */
@Component({
  selector: 'comp-templates',
  templateUrl: 'comp-template.component.html',
  styleUrls: ['comp-template.component.css'],
})
export class CompTemplateComponent implements OnDestroy {
  loading = 0;
  templateElement;

  routeQueryParams$: Subscription;
  queryParamTemplate;

  mobileQuery: MediaQueryList;

  infoDialog:string =
  "<p>"+
      "Browse available component template definitions."+
  "<p>";

  private _mobileQueryListener: () => void;

  constructor(changeDetectorRef: ChangeDetectorRef,
    media: MediaMatcher,
    public dialog: MatDialog,
    private compTemplatesService: CompTemplatesService,
    private route: ActivatedRoute,
    private router:Router,
    private titleService: Title)
  {
    titleService.setTitle('Component Templates');
    this.mobileQuery = media.matchMedia('(max-width: 600px)');
    this._mobileQueryListener = () => changeDetectorRef.detectChanges();
    this.mobileQuery.addListener(this._mobileQueryListener);
    this.routeQueryParams$ = route.queryParams.subscribe(params => {
      if (params['template']) {
        this.queryParamTemplate=params['template'];
      }
    });
  }

  ngOnDestroy(): void {
    this.mobileQuery.removeListener(this._mobileQueryListener);
  }


  openInfoDialog(): void{
    const dialogRef = this.dialog.open(InfoDialogComponent,
      {
      width: '40%',
      height: '40%',
      data: {
        type: "comp-template",
        text: this.infoDialog
      }
      }
    );
  }

  ngOnInit():void {
    this.templateElement = document.getElementById('template');
    this.loading++;
    this.compTemplatesService.getCompTemplate(this.queryParamTemplate)
      .subscribe(
        (data) => {
          this.templateElement.value = data['result']
          this.loading--;
        },
        (error) => {
          this.loading--;
          this.templateElement.innerHTML = 'Error loading component template';
        }
      );

  }

}
