import { Component, OnInit, Inject } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { ConfigBuilderService } from '../../services/config-builder.service'; 
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';

@Component({
  selector: 'app-init-dialog',
  templateUrl: './init-dialog.component.html',
  styleUrls: ['./init-dialog.component.css']
})
export class InitDialogComponent implements OnInit {
  loading = 0; 
  errors='';
  init_config_value='';
  
  initConfigForm = new FormGroup({
    repoURL: new FormControl('',[Validators.required, Validators.pattern('http(s?)\:\/\/[0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*(:(0-9)*)*(\/?)([a-zA-Z0-9\-\.\?\,\'\/\\\+&amp;%\$#_]*)?\.git')]),
    ref: new FormControl('master',[Validators.required])
  })
  constructor( public dialogRef: MatDialogRef<InitDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data,
     private configBuilderService: ConfigBuilderService) { 
  }

  

  get repoURL(): any{
    return this.initConfigForm.get('repoURL');
  }
  get ref(): any{
    return this.initConfigForm.get('ref');
  }

  ngOnInit(): void {
  }

  initialize(source_repo_url, source_repo_ref) {
    this.errors = '';

    this.loading++;
    this.dialogRef.disableClose=true;
    this.configBuilderService.initConfig(source_repo_url, source_repo_ref, null).
      subscribe(
        (data) => {
          if(data['errors'].length) {
            this.errors = data['errors'].join("\n-> ");
          } else {
            this.init_config_value = data['result'];
            this.errors='';
            this.dialogRef.close(
              {
                data: this.init_config_value,
                env: source_repo_ref,
                repoURL: source_repo_url
              }
            );
          }
          this.loading--;
          this.dialogRef.disableClose=false;
      },
      (error) =>{
        this.errors = 'Unexpected error in trying to initialize config';
        this.loading--;
        this.dialogRef.disableClose=false;
      }
    );
  };

}
