import {MediaMatcher} from '@angular/cdk/layout';
import {ChangeDetectorRef, Component, OnDestroy} from '@angular/core';
import { InfoDialogComponent } from '../info-dialog/info-dialog.component';
import { MatDialog } from '@angular/material/dialog';
import { GeneralService } from '../../app/services/general.service';
import { DocsService } from '../../app/services/docs.service';
import { Title } from '@angular/platform-browser';



@Component({
  selector: 'app-docs',
  templateUrl: './docs.component.html',
  styleUrls: ['./docs.component.css']
})
export class DocsComponent implements OnDestroy {
  jQuery;
  componentVersion='';
  sourceVersion='';
  integrations='';
  error='';
  loading=0;

  mobileQuery: MediaQueryList;
  private _mobileQueryListener: () => void;

  infoDialog:string = 
  "<p>"+
      "Documentation and general information for this f8 instance."+
  "</p>";

  constructor(changeDetectorRef: ChangeDetectorRef,
    media: MediaMatcher,
    public dialog: MatDialog,
    private generalService: GeneralService,
    private docsService: DocsService,
    private titleService: Title)
  {
    titleService.setTitle('Docs');
    this.mobileQuery = media.matchMedia('(max-width: 600px)');
    this._mobileQueryListener = () => changeDetectorRef.detectChanges();
    this.mobileQuery.addListener(this._mobileQueryListener);
  }

  ngOnDestroy(): void {
    this.mobileQuery.removeListener(this._mobileQueryListener);

  }


  openInfoDialog(): void{
    const dialogRef = this.dialog.open(InfoDialogComponent, 
      {
      width: '40%',
      height: '40%',
      data: {
        type:"docs",
        text:this.infoDialog
      }
      }
    );
  }

  ngOnInit():void {
    this.loading++;
    this.jQuery = this.generalService.returnJQuery();
    var fragment = window.location.hash.substr(1);
    if (fragment) this.jQuery('#' + fragment).toggleClass('collapse');
    this.docsService.getInfo()
      .subscribe(
        (data) => {
          this.componentVersion = data['container_version'];
          this.sourceVersion = data['source_version'];
          this.integrations = data['integrations'];
          this.error='';
          this.loading--;
        },
        (error) => {
          this.loading--;
          this.error = 'Error loading version and system information.'
        }
      )

  }

}
