import { Component, OnInit, Input } from '@angular/core';
import { MatTableDataSource } from  '@angular/material/table';
import { GeneralService } from '../../services/general.service';
import { EventsService } from '../../services/events.service';

@Component({
  selector: 'app-env-graph',
  templateUrl: './env-graph.component.html',
  styleUrls: ['./env-graph.component.css']
})
export class EnvGraphComponent implements OnInit {


  detailsSource = new MatTableDataSource([]);
  detailsDisplayedColumns:string[] = ['name', 'type', 'cluster', 'group', 'tag'];

  instanceSource = new MatTableDataSource([]);
  instanceDisplayedColumns:string[] = ['id', 'status', 'healthy'];

  @Input() envDetails;
  @Input() omitDetailTable;
  oldEnvDetails; //used to check if response is different 


  constructor(private generalService: GeneralService) { }

  ngOnInit(): void {
    this.oldEnvDetails = {...this.envDetails};
    if(!this.omitDetailTable){
      this.populateDetailsTable();
      this.populateInstanceTable();
    }
    this.createGraph();


  }

  ngOnChanges():void {
    if (!(JSON.stringify(this.oldEnvDetails)===JSON.stringify(this.envDetails))){ //only update if there are changes 
      this.oldEnvDetails = {...this.envDetails}
      this.populateDetailsTable();
      this.populateInstanceTable();
      this.createGraph();
    }
  }

  populateDetailsTable(){
    var detailsData = [];
    var detailsObject = {};
    detailsObject['name'] = this.envDetails['name'];
    detailsObject['type'] = this.envDetails['type'];
    detailsObject['cluster'] = this.envDetails['cluster'];
    detailsObject['group'] = this.envDetails['group'];
    detailsObject['tag'] = this.envDetails['tag'];
    detailsData.push(detailsObject);

    this.detailsSource.data=detailsData;
  }

  populateInstanceTable(){
    var instanceData = [];
    for (var i in this.envDetails.components){
      var comp = this.envDetails.components[i];
      for (var j in comp.instances){
        var inst = comp.instances[j];
        //console.log(inst);
      
        if (inst['state'] && inst['state']['exists']){
          var instanceObj = {};
          instanceObj['id']=inst['name'];
          instanceObj['status']=inst['state']['name'];
          instanceObj['healthy']=inst['state']['healthy'];
          instanceData.push(instanceObj);
        }
      }
    }
    for (var i in this.envDetails.tests){
      var test = this.envDetails.tests[i];
      for (var j in test.instances){
        var inst = test.instances[j];
        //console.log(inst);
      
        if (inst['state'] && inst['state']['exists']){
          var instanceObj = {};
          instanceObj['id']=inst['name'];
          instanceObj['status']=inst['state']['name'];
          instanceObj['healthy']=inst['state']['healthy'];
          instanceData.push(instanceObj);
        }
      }
    }

    this.instanceSource.data=instanceData;

  }

  createGraph(){
    //this.eventsService.getEvents(status['service'], status['name'], status['cluster'], d_name).subscribe((even) => {

      this.generalService.createNetworkGraph(this.envDetails);
  }

  

  
}
