import { Component, OnInit, Input, OnDestroy } from '@angular/core';
import { GeneralService } from 'src/app/services/general.service';
import { Globals } from '../../global'
import {EnvironmentsService} from '../../services/environments.service'
import { Subject, Subscription } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { HttpEventType, HttpDownloadProgressEvent, HttpResponse } from '@angular/common/http';
import { NgControlStatus } from '@angular/forms';
//import { runInThisContext } from 'vm';


@Component({
  selector: 'app-env-log',
  templateUrl: './env-log.component.html',
  styleUrls: ['./env-log.component.css']
})


export class EnvLogComponent implements OnInit, OnDestroy{
  protected ngUnsubscribe: Subject<void> = new Subject<void>();

  objectKeys = Object.keys;


  @Input() envDetails; 
  @Input() envString;
  @Input() runScroll; //used to trigger onChanges every time (for log scrolling)
  oldEnvDetails = {...this.envDetails}; //used to check if response is different 

  //used one time for initial logs - deleted afterwards 
  //@Input() queryParamShow = 'logs';
  @Input() queryParamAutostart = false;
  @Input() queryParamCompsAndInstances = null //{name: isChild} ex {'db': false}
  

  nodes = {}; //model for instances
  childrenToParent = {} //{instanceName:ParentName} - reverse mapping used for query Parameter efficiency 
  // {
  // db : {
  //   id: idCount,
  //   name:component.name,
  //   parentSelected:false,
  //   children: [
  //       { 
  //         id: idCount,
  //         childSelected:false,
  //         name:instance.name
  //       },
  //       {...}
  //     ]
  //   }
  // }

  //log_instances:string[];
  log_subscription:Subscription = null;
  //log_request = null;
  log_display;
  error_display; 
  logs_since_seconds = '300';
  log_timestamps

  loading=0;


  selectedInstancesObj={}; //{name:isChild}
  

  //jQuery;
  disabled:boolean = true;

  errors=[];



  constructor(private generalService: GeneralService,
    private globals: Globals,
    private environmentService: EnvironmentsService) { }

  public ngOnDestroy(): void {
      // This aborts all HTTP requests.
      this.ngUnsubscribe.next();
      // This completes the subject properlly.
      this.ngUnsubscribe.complete();
    }
  
  ngOnChanges(){
    //This will kick off every time, due to logTabScroll field in environments component:
    this.alterCompsAndInstancesQueryParams();
    if (this.log_display){
      this.generalService.scrollToBottom(this.log_display);
    }
    if (this.error_display){
      this.generalService.scrollToBottom(this.error_display);
    }
    if (!(JSON.stringify(this.oldEnvDetails)===JSON.stringify(this.envDetails))){ //only update if there are changes 

      //should clear out this.nodes (this.nodes={}), if the application(or environment?) is different
      this.nodes={};
      //clear out the children to parent mapping as well 
      this.childrenToParent = {};
       //otherwise, will need to edit this.nodes, instead of clearing it
      this.loadComponentList(this.nodes);

      //if we change selection
      if (this.oldEnvDetails.name !== this.envDetails.name ||
        this.oldEnvDetails.service !== this.envDetails.service ||
        this.oldEnvDetails.cluster !== this.envDetails.cluster) {

          //kill existing log
          if (this.log_subscription){ 
            this.log_subscription.unsubscribe();
            // if (this.errors.length>0) {
            //   this.errors=[];
            // }
            this.error_display.innerHTML = '';
            this.log_display.innerHTML = '';
            this.loading=0;
          }
          this.selectedInstancesObj={}; 

        } else { //instances updated 
          var newSelectedInstancesObj = {};
          for (var parent in this.nodes){
            if (parent in this.selectedInstancesObj){
              this.nodes[parent]['parentSelected'] = true;
              newSelectedInstancesObj[parent] = false;
              //set all children to true
              for (var c in this.nodes[parent]['children']){
                this.nodes[parent]['children'][c]['childSelected'] = true;
                newSelectedInstancesObj[this.nodes[parent]['children'][c]['name']] = true;
              }
            } else {
              //set all children to true if in selectedInstancesObj
              for (var c in this.nodes[parent]['children']){
                if (this.nodes[parent]['children'][c]['name'] in this.selectedInstancesObj){
                  this.nodes[parent]['children'][c]['childSelected'] = true;
                  newSelectedInstancesObj[this.nodes[parent]['children'][c]['name']] = true;
                }
              }
            }
          }
          this.selectedInstancesObj = {...newSelectedInstancesObj}; 
        }
        this.alterCompsAndInstancesQueryParams();
        this.oldEnvDetails = {...this.envDetails}

      

    }
  }

  loadComponentList(nodeObject){
    


    var idCount:number = 0;
    for (var c in this.envDetails.components){
      idCount++;
      var component = this.envDetails.components[c];
      var parentObj = {
        id: idCount,
        name:component.name,
        parentSelected:false,
        children: []
      }
      for (var inst in component.instances){
        idCount++;
        var instance = component.instances[inst];
        var childObj = {
          id: idCount,
          childSelected:false,
          name:instance.name,
          //parentName:component.name
        }
        parentObj.children.push(childObj);
        this.childrenToParent[instance.name] = component.name;
      }
      nodeObject[component.name]=parentObj;
    }
    for (var t in this.envDetails.tests){
      idCount++;
      var test = this.envDetails.tests[t];
      var parentObj = {
        id: idCount,
        name:test.name,
        parentSelected:false,
        children: []
      }
      for (var inst in test.instances){
        idCount++;
        var instance = test.instances[inst];
        var childObj = {
          id: idCount,
          childSelected:false,
          name:instance.name,
          //parentName:test.name
        }
        parentObj.children.push(childObj);
        this.childrenToParent[instance.name] = test.name;

      }
      nodeObject[test.name]=parentObj;
    }
  }

  // loopThroughSelections(){
  //   this.selectedInstances=[];
  //   for (var p in this.nodes){
  //     var parent = this.nodes[p];
  //     if (parent.parentSelected){
  //       this.selectedInstances.push({
  //         name:parent.name,
  //         isParent:true
  //       });
  //     }

  //     for (var c in parent.children){
  //       var child = parent.children[c];
  //       if (child.childSelected){
  //         this.selectedInstances.push({
  //           name:child.name,
  //           isParent:false
  //         })
  //       }
  //     }
  //   }
  // }

  // submitForm(){
  //   // this.selectedInstances=[];
  //   // for (var p in this.nodes){
  //   //   var parent = this.nodes[p];
  //   //   if (parent.parentSelected){
  //   //     this.selectedInstances.push({
  //   //       name:parent.name,
  //   //       isParent:true
  //   //     });
  //   //   }

  //   //   for (var c in parent.children){
  //   //     var child = parent.children[c];
  //   //     if (child.childSelected){
  //   //       this.selectedInstances.push({
  //   //         name:child.name,
  //   //         isParent:false
  //   //       })
  //   //     }
  //   //   }
  //   // }
  //   console.log(this.selectedInstancesObj)

  // }

  ngOnInit(): void {
    this.log_display = document.getElementById('log-display');
    this.error_display = document.getElementById('error-display');
    this.loadComponentList(this.nodes);
    if (this.queryParamCompsAndInstances !== null){
      this.handleQueryLog();
    }

    //this.jQuery = this.generalService.returnJQuery();
    //this.generalService.validateLogSubmit();
  };

  // ngAfterViewInit() { //needs to be in after View Init so that ngFor have time to load 
  //   this.generalService.validateLogSubmit();
  // }


  alterCompsAndInstancesQueryParams(){
    var compString:String = '';
    var instString:String = '';
    for (var instance in this.selectedInstancesObj){
      if (!this.selectedInstancesObj[instance]){ //if parent
        compString += instance + ',';
      } else if (!(this.childrenToParent[instance] in this.selectedInstancesObj)){
        instString += instance + ',';
      }
    }
    if (compString.length>0) {compString = compString.slice(0, -1);};
    if (instString.length>0) {instString = instString.slice(0, -1);};

    this.generalService.alterQueryParameters({
      comps: compString.length > 0 ? compString : null,
      instances: instString.length > 0 ? instString : null,
    })
  }

  //handling checkboxes:
  checkUncheckAll(parentName){
    // console.log(parentName);
    // console.log(this.nodes)
    if (this.nodes[parentName].parentSelected){
      this.selectedInstancesObj[parentName] = false;
    } else {
      delete this.selectedInstancesObj[parentName];
    }
    for (var i in this.nodes[parentName].children){
      this.nodes[parentName].children[i].childSelected = this.nodes[parentName].parentSelected;
      if (this.nodes[parentName].parentSelected){
        this.selectedInstancesObj[this.nodes[parentName].children[i].name] = true;
      } else {
        delete this.selectedInstancesObj[this.nodes[parentName].children[i].name];
      }
    }
    this.alterCompsAndInstancesQueryParams();
  }

  isAllSelected(parentName){
    // this.nodes[parentName].parentSelected = this.nodes[parentName].children.every(
    //   (item:any) => {
    //     if (item.childSelected){
    //       this.selectedInstancesObj[item.name]=false;
    //     } else {
    //       delete this.selectedInstancesObj[item.name];
    //     }
    //     return item.childSelected == true;
    //   }
    // )
    this.nodes[parentName].parentSelected = true;
    for (var i in this.nodes[parentName].children){
      if (!this.nodes[parentName].children[i].childSelected){
        this.nodes[parentName].parentSelected = false;
        delete this.selectedInstancesObj[this.nodes[parentName].children[i].name]
      } else {
        this.selectedInstancesObj[this.nodes[parentName].children[i].name] = true;
      }
    }
    
    if (this.nodes[parentName].parentSelected){
      this.selectedInstancesObj[parentName]=false;
    } else {
      delete this.selectedInstancesObj[parentName];
    }
    this.alterCompsAndInstancesQueryParams();
  }

   // {
  // db : {
  //   id: idCount,
  //   name:component.name,
  //   parentSelected:false,
  //   children: [
  //       { 
  //         id: idCount,
  //         childSelected:false,
  //         name:instance.name
  //       },
  //       {...}
  //     ]
  //   }
  // }


  handleQueryLog(){
    if (this.queryParamCompsAndInstances !== null ){
      for (var parentName in this.nodes){
        if (parentName in this.queryParamCompsAndInstances){

          this.selectedInstancesObj[parentName] = false;
          this.nodes[parentName].parentSelected = true;

          //loop through the children
          for (var i in this.nodes[parentName].children){
            this.selectedInstancesObj[this.nodes[parentName].children[i].name] = true;
            this.nodes[parentName].children[i].childSelected = true;
          }
        } else {
          //assume parent is selected until a child not selected
          this.selectedInstancesObj[parentName] = false;
          this.nodes[parentName].parentSelected = true;
          //if no children, mark as false
          if (this.nodes[parentName].children.length==0){
            this.nodes[parentName].parentSelected = false;
            delete this.selectedInstancesObj[parentName];
          }
          //loop through the children
          for (var i in this.nodes[parentName].children){
            if (this.nodes[parentName].children[i].name in this.queryParamCompsAndInstances){
              this.selectedInstancesObj[this.nodes[parentName].children[i].name] = true;
              this.nodes[parentName].children[i].childSelected = true;
            } else {
              delete this.selectedInstancesObj[parentName];
              this.nodes[parentName].parentSelected = false;
            }
          }
        }
      }
      this.alterCompsAndInstancesQueryParams();
      //null this out so query parameter is only handled once 
      this.queryParamCompsAndInstances = null;
      if (this.queryParamAutostart){
        this.get_logs();
      }
    }
  }

  getSelectedComps(){
    var selectedComps = [];
    for (var parentName in this.nodes){
      if (this.nodes[parentName].parentSelected) selectedComps.push(parentName);
    }
    return selectedComps;
  }

  save_log(){
    var splits = this.generalService.split_env_string(this.envString);
    var log_name = "logs-"+splits[0]+"-"+splits[1]+"-"+this.generalService.getDateInUTC() + "-UTC";
    console.log('saving file: ' + log_name);
    this.generalService.saveTextAsFile(this.log_display.innerText, log_name);
  }

  // validateSubmitButton(){
    
  //   return this.jQuery(".env-log-tree input:checkbox:checked").length > 0
  // }

  // count(){
  //   this.checkedTotal++
  //   console.log(this.checkedTotal);
  // }

  get_logs() {

    var NOTICE_START_IDENTIFIER = 'NOTICE:';
    var NOTICE_END_IDENTIFIER = '<br/>';

    this.loading=0;
    this.loading++;
    if (this.log_subscription){
      this.log_subscription.unsubscribe();
    }
    //if (this.log_request) this.log_request.abort();
    this.error_display.innerHTML=''
    this.log_display.innerHTML = '';
    var last_index = 0;
    //var xhr = new XMLHttpRequest();
    //this.log_request = xhr;
    var splits = this.generalService.split_env_string(this.envString);
    var service = splits[0];
    var env_name = splits[1];
    var selected_cluster = splits[2];
    
    var log_instances =[];
    for (var instance in this.selectedInstancesObj){
      if (this.selectedInstancesObj[instance]){
        log_instances.push(instance);
      }
    }
        
    // if (parseInt(this.logs_since_seconds)) url += '&since_seconds=' + this.logs_since_seconds;
    // if (this.log_timestamps) url += '&timestamps=true';
     this.log_subscription = this.environmentService.getEnvironmentLogs(service, env_name, log_instances, selected_cluster, this.logs_since_seconds, this.log_timestamps, this.getSelectedComps())
      .pipe( takeUntil(this.ngUnsubscribe) )
      .subscribe(
        (event) => {
          if (event.type === HttpEventType.DownloadProgress){
            const responseText = (<HttpDownloadProgressEvent>event).partialText;
            if (last_index===0){
              this.loading--; // if first response then disable loading gif
            } 
            var curr_index = responseText.length;
            if (last_index == curr_index){
              return
            }
            var s = responseText.substring(last_index, curr_index);
            last_index = curr_index;
            // check scroll location before adding new logs
            var isScrolledToBottom = this.log_display.scrollHeight - this.log_display.clientHeight <= this.log_display.scrollTop + 1;
            if (s.trim().substring(0, 7) == NOTICE_START_IDENTIFIER) {
              var msg = s.substring(8, s.indexOf(NOTICE_END_IDENTIFIER) + NOTICE_END_IDENTIFIER.length);
              var date = this.generalService.getDateInUTC();
              var errorIsScrolledToBottom = this.error_display.scrollHeight - this.error_display.clientHeight <= this.error_display.scrollTop + 1;
              this.error_display.innerHTML += "<div style='padding-bottom:2px; padding-left:2px;'>"+ date + "&nbsp; " + msg + "</div>"
              if (errorIsScrolledToBottom){
                this.generalService.scrollToBottom(this.error_display);
              }
            }
            this.log_display.innerHTML += s;
            if (isScrolledToBottom) {
              this.generalService.scrollToBottom(this.log_display);
            }
          } else if (event instanceof HttpResponse) {
              this.log_display.innerHTML += '--- End of logs for the selected instances ---';
              //if scroll location was bottom, then reset it there
              var isScrolledToBottom = this.log_display.scrollHeight - this.log_display.clientHeight <= this.log_display.scrollTop + 1;
              
              if (isScrolledToBottom){
                this.generalService.scrollToBottom(this.log_display);
              }
          }
        },
        (error) => {
          console.log(error);
          var msg = 'overall log request terminated, click Start to restart';
          var date = this.generalService.getDateInUTC();
          var errorIsScrolledToBottom = this.error_display.scrollHeight - this.error_display.clientHeight <= this.error_display.scrollTop + 1;
          this.error_display.innerHTML += "<div style='color:red; padding-bottom:2px; padding-left:2px'>"+ date + "&nbsp; " + msg + "</div>"
          if (errorIsScrolledToBottom){
            this.generalService.scrollToBottom(this.error_display);
          }
          

          this.loading--;
          
          if (this.loading < 0) {
          this.loading = 0;
          }
        }
      )
};
  
}
