import {MediaMatcher} from '@angular/cdk/layout';
import {ChangeDetectorRef, Component, OnDestroy} from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { InfoDialogComponent } from '../info-dialog/info-dialog.component';
import { GeneralService } from '../services/general.service';
import { EnvironmentsService } from '../services/environments.service';
import { finalize } from 'rxjs/operators';
import { Globals } from '../global';
import { takeUntil } from 'rxjs/operators'; //needed to unsubscribe from all, when navigating away
import { Subject, Subscription } from 'rxjs';
import { ActivatedRoute, Router } from '@angular/router';
import { Title } from '@angular/platform-browser';





@Component({
  selector: 'app-environments',
  templateUrl: './environments.component.html',
  styleUrls: ['./environments.component.css']
})
export class EnvironmentsComponent implements OnDestroy {
  protected ngUnsubscribe: Subject<void> = new Subject<void>();

  infoDialog:string = 
  "<span>"+ //must be span for environments, to work with JSONEditor library <p> tag shrinkage 
  "This page can be used to view details on all environments deployed with f8."+
  "<ul>"+
    "<li>Once an environment is selected from the first dropdown menu, its status, configuration, and logs become available. </li>"+
    "<li>Environment status can be updated by clicking the refresh button, as well as automatically at the interval chosen in the second dropdown menu (not available on mobile screens).</li>"+
    "<li>Externally accessible components are denoted by a globe, which opens their URL when clicked.</li>"+
  "</ul>"+
  "</span>";
 

  routeQueryParams$: Subscription;
  queryParamApp
  queryParamEnv
  queryParamCluster

  queryParamShow

  queryParamAutoStart
  logsAutoStart = false;

  queryParamComps
  queryParamInstances
  logsCompsAndInstances=null;


  mobileQuery: MediaQueryList;
  loading=0;
  errors={};
  servingEnv;

  jQuery;


  env_strings=[];
  env_string='';
  appPiece='';
  envPiece='';
  clusterPiece=''

  env_refresh_interval="300";
  env; //the details payload
  envDropDownElement;
  
  //config builder for this env 
  env_config;


  //log
  logDisplayElement;
  logTabScroll = false;
  //log_request = null;

  //refresh timer
  refresh_timer_id;



  //navigation between components
  tab = 1;
 

  
  private _mobileQueryListener: () => void;

  constructor(changeDetectorRef: ChangeDetectorRef, 
    media: MediaMatcher,
    public dialog: MatDialog,
    private generalService: GeneralService,
    private environmentsService: EnvironmentsService,
    private global: Globals,
    private route: ActivatedRoute,
    private router: Router,
    private titleService: Title) {
      
    titleService.setTitle('Environments');
    this.mobileQuery = media.matchMedia('(max-width: 600px)');
    this._mobileQueryListener = () => changeDetectorRef.detectChanges();
    this.mobileQuery.addListener(this._mobileQueryListener);
    this.routeQueryParams$ = route.queryParams.subscribe(params => {
      if (params['app']) {
        this.queryParamApp=params['app'];
      }
      if (params['env']) {
        this.queryParamEnv=params['env'];
      }
      if (params['cluster']) {
        this.queryParamCluster=params['cluster'];
      }
      if (params['show']) {
        this.queryParamShow = params['show'];
      }
      if (params['autostart']) {
        this.queryParamAutoStart = params['autostart'];
      }
      if (params['comps']) {
        this.queryParamComps = params['comps'];
      }
      if (params['instances']) {
        this.queryParamInstances = params['instances'];
      }
    });
  }



  ngOnInit() {
    //open info dialog: 
    //this.openInfoDialog();
    this.generalService.initEnvToolbarTimeList()
    // if(screen.width < 768){
     
    //   document.getElementById('env-picker').title = "Env";
    // } 

    if (this.queryParamApp && this.queryParamEnv && this.queryParamCluster){
      this.env_string = this.queryParamApp+'/'+this.queryParamEnv+' ('+this.queryParamCluster+')';
      if (this.queryParamShow === 'status') {
        this.setTab(1);
      } else if (this.queryParamShow === 'configuration') {
        this.setTab(2);
      } else if (this.queryParamShow === 'logs') {
        this.setTab(3);
      } 
      if (this.queryParamAutoStart === 'true') {
        this.logsAutoStart = true;
      } else {
        this.logsAutoStart = false;
      }

      if (this.queryParamComps) {
        if (this.logsCompsAndInstances === null){
          this.logsCompsAndInstances = {};
        }
        var compsArray = this.queryParamComps.split(',');
        for (var c in compsArray){
          var comps = compsArray[c];
          this.logsCompsAndInstances[comps] = false;
        }
      }

      if (this.queryParamInstances) {
        if (this.logsCompsAndInstances === null){
          this.logsCompsAndInstances = {};
        }
        var instArray = this.queryParamInstances.split(',');
        for (var i in instArray){
          var inst = instArray[i];
          this.logsCompsAndInstances[inst] = true;
        }
      }

      this.update_env();
     } else {
      this.openInfoDialog();
     }
    //initialize list of Environments
    //TODO: when you have query parameters, probably
    //need to await initializeList() to finish, then run update_env() with query params
    //this.envListElement = document.getElementById('env-drop-down');
    this.jQuery = this.generalService.returnJQuery();
    this.envDropDownElement = document.getElementById('env-picker');
    this.logDisplayElement = document.getElementById('log-display');
    this.initializeList(true);
    this.loading++;
    this.environmentsService.getSchemas()
      .pipe( takeUntil(this.ngUnsubscribe) )
      .subscribe(
        (data)=> {
          try {
            this.env_config = this.generalService.create_form_environments('env-config', data, {}, undefined);
            this.generalService.shrink_pTag('13px');
            this.loading--;
            delete this.errors['env'];
          } catch {
           
            this.errors['env']='Error populating configuration. Please refresh the page.';
            this.loading--;
            // console.log("hit try catch 1");
          }
          
          //add call to update envs if query params
        },
        (error)=> {
          this.errors['env']='Error populating configuration. Please refresh the page.';
          this.loading--;
        }
      )
  }



  initializeList(firstTime:boolean){
    this.loading++;
    
    this.environmentsService.getEnvironmentList()
      .pipe( takeUntil(this.ngUnsubscribe) )
      .subscribe(
          async (data)=> {
          this.servingEnv = data['meta']['serving_env'];
          var envArray=[];
          //envArray.push({id:0, text:''});
          for (var i = 0; i<data['results'].length; i++){
            var tempEnvString:string = data['results'][i]['service'] + '/' + data['results'][i]['name'];
            if (data['results'][i]['cluster']){
              tempEnvString += ' ('+data['results'][i]['cluster']+')';
            }
            // envArray.push({id:i+1, text:tempEnvString});
            envArray.push(tempEnvString);
          }
          this.env_strings = envArray;
          //this.generalService.updateBootstrapSelectLibrary();
          if (firstTime){ 
            this.set_env_refresh();
            this.generalService.initEnvToolbarList();
            if (this.queryParamApp && this.queryParamEnv && this.queryParamCluster){
              var tempEnvString = this.queryParamApp+'/'+this.queryParamEnv+' ('+this.queryParamCluster+')';
              await this.delay(1);
              this.generalService.setSelectedEnv(tempEnvString);
              // this.jQuery('.env-select').selectpicker('val', tempEnvString);
            }
          } else {
            //this.update_list();
            //add delay 
            await this.delay(1);
            this.generalService.updateEnvlist();
            //this.jQuery('.env-select').selectpicker('val', 'test90');
            //this.env_string = this.envDropDownElement.value;  
            //console.log(this.env_string);
          }
          // this.envDropDownElement = this.generalService.getSelect2Element();
          // this.jQuery('#env-drop-down').on("select2:select", this.change_env());
          this.loading--;
          delete this.errors['env'];
          
        },
        (error) => {
          this.errors['env']=this.global.ENV_LIST_ERROR;
          this.loading--;
        }
      )
  }



  ngOnDestroy(): void {
    this.mobileQuery.removeListener(this._mobileQueryListener);
    this.routeQueryParams$.unsubscribe();

    if (this.refresh_timer_id) {
      clearInterval(this.refresh_timer_id);
    }
    
    // This aborts all HTTP requests.
    this.ngUnsubscribe.next();
    // This completes the subject properlly.
    this.ngUnsubscribe.complete();
  }
  

  // split_env_string(s){
  //   var parts = s.split(' ');
  //   var app_string = parts[0].split('/')[0];
  //   var env_string = parts[0].split('/')[1];
  //   var cluster_string = parts.length > 1 ? parts[1].substr(1, parts[1].length - 2) : '';
  //   return [app_string, env_string, cluster_string];
  // }

  // addQueryParameters() {
   
  //   //this.removeQueryParameters();

  //   this.router.navigate([], {
  //    queryParams: {
  //      app: this.appPiece,
  //      env: this.envPiece,
  //      cluster: this.clusterPiece,
  //    },
  //    queryParamsHandling: 'merge',
  //  });
  // }

  // removeQueryParameters() {
  //   this.router.navigate([], {
  //     queryParams: {
  //       app: null,
  //       env: null,
  //       cluster: null,
  //       show:null,
  //       autostart:null,
  //       comps:null,
  //       instances:null
  //     },
  //     queryParamsHandling: 'merge'
  //   })
  // }

  update_env(){
    if (this.env_string){
      var splits = this.generalService.split_env_string(this.env_string);

      var aPiece = splits[0];
      var ePiece = splits[1];
      var cPiece = splits[2];

      if (this.appPiece==='' && this.envPiece==='' && this.clusterPiece===''){
        this.appPiece = aPiece;
        this.envPiece = ePiece;
        this.clusterPiece = cPiece;
        //this.addQueryParameters();
        this.generalService.alterQueryParameters({
          app: this.appPiece,
          env: this.envPiece,
          cluster: this.clusterPiece,
        });
      }
      this.loading++;

      this.environmentsService.getEnvironmentDetails(aPiece, ePiece, cPiece)
        .pipe( takeUntil(this.ngUnsubscribe) )
        .subscribe(
          (data) =>{
            this.env = data['results'][0];
            this.appPiece = aPiece;
            this.envPiece = ePiece;
            this.clusterPiece = cPiece; 
            //this.addQueryParameters();
            //this.setTabQueryParameter(this.tab);
            this.generalService.alterQueryParameters({
              app: this.appPiece,
              env: this.envPiece,
              cluster: this.clusterPiece,
              show: this.tab == 1 ? 'status' : this.tab == 2 ? 'configuration' : this.tab == 3 ? 'logs' : null
            });
            
//          document.getElementById('header').innerHTML = $scope.env_string;

            //config:
            try {
              var cfg_obj = Object.assign({}, this.env['cfg']);
              delete cfg_obj['env-types']; // if showing env config, env-types doesn't make sense really
              this.env_config.setValue(cfg_obj);
              this.env_config.disable();
              this.generalService.shrink_pTag('13px');
              this.loading--;
              delete this.errors['cfg'];
              
            } catch {
              
              this.env = {config: {}, status: {}};
              
              this.loading--;
              console.log("Error populating configuration. Please refresh the page.");
              this.errors['cfg'] = 'Error populating configuration. Please refresh the page.'
            }
            
          },
          (error)=>{
            // this.generalService.alterQueryParameters({
            //   app: this.appPiece,
            //   env: this.envPiece,
            //   cluster: this.clusterPiece,
            // });
            // this.setTabQueryParameter(this.tab);

            this.errors['cfg'] = 'could not find environment ' + this.env_string;
            this.env = {config: {}, status: {}};

            //config:
            this.loading--;
          } 
        )
    }
    // else {
    //   this.removeQueryParameters();
    // }
  }

  change_env(){
    this.env_string = this.envDropDownElement.value;
    // this.generalService.getSelectedEnv();
    // console.log(this.env_string);
    // if (this.log_request) {
    //   this.log_request.abort();
    // }
    // console.log(this.env_string);
    //this.logDisplayElement.innerHTML = ''; comment out for now, since have not implemented logs yet 
    //this.removeQueryParameters();
    this.generalService.alterQueryParameters({
      app: null,
      env: null,
      cluster: null,
      show:null,
      autostart:null,
      comps:null,
      instances:null
    });
    this.update_env();
    
  } 

delay(ms: number) {
    return new Promise( resolve => setTimeout(resolve, ms) );
}


 update_list(){
    for (let i = 0; i<100; i++){
      this.env_strings.push("test"+i);
    }

    // this.jQuery('.env-select').selectpicker('refresh');
    //this.generalService.updateEnvList();
  }

  refresh(){
    this.initializeList(false);
    this.update_env();
  }

  siyTest(text){
    console.log("test");
  }

  set_env_refresh() {
    var freq = 1000*parseInt(this.env_refresh_interval);
    //console.log(freq);
    if (this.refresh_timer_id) {
      clearInterval(this.refresh_timer_id);
    }
    if (freq) {
      this.refresh_timer_id = window.setInterval(()=>this.refresh(), freq);
      
    }
};

  setTabQueryParameter(tabNum) {
    this.generalService.alterQueryParameters({
      show: tabNum == 1 ? 'status' : tabNum == 2 ? 'configuration' : tabNum == 3 ? 'logs' : null,
      instances:null,
      comps:null
    })
    // if (tabNum == 1) {
    //   if (this.queryParamShow) {
    //     //this.generalService.alterQueryParameters({show:null});
    //   } 
    //   this.generalService.alterQueryParameters({show:'status'});
    // } else if (tabNum == 2) {
    //   if (this.queryParamShow) {
    //     //this.generalService.alterQueryParameters({show:null});
    //   } 
    //   this.generalService.alterQueryParameters({show:'configuration'});
    // } else if (tabNum == 3) {
    //   if (this.queryParamShow) {
    //     //this.generalService.alterQueryParameters({show:null});
    //   } 
    //   this.generalService.alterQueryParameters({show:'logs'});
    // } 
  }

  setTab(newTab) {

    // if (newTab != 3) {
    //     if (this.log_request) {
    //       // this.log_request.abort(); TODO: need to implement later 
    //     }
    //     if (this.errors['logs']) {
    //       delete this.errors['logs'];
    //     }
    //     // this.logDisplayElement.innerHTML = ''; TODO: need to implement later
    // }

    if (newTab == 3 ) {
      //focus to bottom of window
      // var element = document.getElementById("log-display");
      //if (element!==null){
        // console.log(element.scrollTop);
        // console.log(element.scrollHeight);
        // console.log(element.clientHeight);
        // element.scrollTop = element.scrollHeight - element.clientHeight;
      //}
      this.logTabScroll = !this.logTabScroll;
    }
    
    
    this.setTabQueryParameter(newTab);

    this.tab = newTab;
};

  isSet(tabNum) {
    // if (tabNum === 1) {
    //   this.generalService.alterQueryParameters({
    //     show:null
    //   })
    //   this.generalService.alterQueryParameters({
    //     show:'status'
    //   })
    // } else if (tabNum === 2 ){
    //   this.generalService.alterQueryParameters({
    //     show:null
    //   })
    //   this.generalService.alterQueryParameters({
    //     show:'configuration'
    //   })
    // } else if (tabNum === 3 ) {
    //   this.generalService.alterQueryParameters({
    //     show:null
    //   })
    //   this.generalService.alterQueryParameters({
    //     show:'logs'
    //   })
    // }
    return this.tab === tabNum;
  };


  openInfoDialog(): void{
    const dialogRef = this.dialog.open(InfoDialogComponent, 
      {
      width: '60%',
      height: '45%',
      data: {
        type:"environments",
        text: this.infoDialog
      }
      }
    );
  }

  checkErrorObjIsEmpty(errObj) {
    return (errObj && (Object.keys(errObj).length === 0));
  }
}
