import { Component, OnInit } from '@angular/core';
import { Title } from '@angular/platform-browser';
import { Router,ActivatedRoute } from '@angular/router';
import { AuthService } from '../services/auth.service';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import * as utils from '../services/utils';
import { environment } from 'src/environments/environment';


@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css']
})
export class LoginComponent implements OnInit {

  username: string;
  password: string;
  showSpinner: boolean;
  successfulLogin: boolean;
  returnUrl: string;
  constructor(
    private titleService: Title,
    private authService: AuthService,
    private http: HttpClient,
    private router: Router,
    private route: ActivatedRoute) {
      titleService.setTitle('Login');
    }
    

  ngOnInit() {
    this.successfulLogin = true;
    this.route.queryParams
      .subscribe(params => {
        this.returnUrl = params['return'] || '/home';
        if(params['token']) {
          //store the login token
          try {
            const env = utils.get_environment();
            localStorage.setItem(`${env}_auth_token`, params['token']);
            this.router.navigateByUrl(this.returnUrl);
          }catch(e){
            localStorage.clear();
          }
        }
      });
  }

  login() {
    this.showSpinner = true;
    this.authService
      .login({ "email": this.username, "password": this.password })
      .subscribe(
        data => {
          this.showSpinner = false;
          //store the login token
          try {
            const env = utils.get_environment();
            localStorage.setItem(`${env}_auth_token`, data["access_token"]);
          }catch(e){
            localStorage.clear();
          }
          this.router.navigateByUrl(this.returnUrl);
        },
        (err: HttpErrorResponse) => {
          this.successfulLogin = false;
          this.showSpinner = false;
        });
  }

  githubLogin() {
    this.showSpinner = true;
    if(environment.production) {
      window.location.href = '../../auth-api/8081/auth/login_github';
    } else {
      window.location.href = 'http://localhost:8081/auth/login_github';
    }
  }

}
