import { Injectable, Inject } from '@angular/core';
import { HttpClient, HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import  * as utils from './utils';
import { environment } from 'src/environments/environment';
import { User } from '../models/user';
import { Observable } from 'rxjs';


@Injectable({
  providedIn: 'root'
})
export class AuthService {

  readonly TOKEN_KEY = "auth_token";

  constructor( @Inject('API_URL') private apiUrl : string, private httpClient : HttpClient) { }

  login(params) {
    localStorage.removeItem("auth_token");
    const reqHeaders = new HttpHeaders();
    reqHeaders.set('Content-Type', 'application/json; charset=utf-8');
    return this.httpClient.post("/auth/login", params, {headers: reqHeaders});
  }

  createUser(params) {
    const reqHeaders = new HttpHeaders();
    reqHeaders.set('Content-Type', 'application/json; charset=utf-8');
    return this.httpClient.post("/auth/users/", params, {headers: reqHeaders});
  }

  githubLogin() {
    localStorage.removeItem("auth_token");
    const reqHeaders = new HttpHeaders();
    reqHeaders.set('Content-Type', 'application/json; charset=utf-8');
    return this.httpClient.get('/auth/login_github',  {headers: reqHeaders});

  }

  editUser(old_email: string, params) {
    const reqHeaders = new HttpHeaders();
    reqHeaders.set('Content-Type', 'application/json; charset=utf-8');
    return this.httpClient.put(`/auth/users/${old_email}/edit`, params, {headers: reqHeaders});
  }

  getUsers() {
    const reqHeaders = new HttpHeaders();
    reqHeaders.set('Content-Type', 'application/json; charset=utf-8');
    return this.httpClient.get('/auth/users/', {headers: reqHeaders});
  }

  deleteUser(email : string) {
    const reqHeaders = new HttpHeaders();
    reqHeaders.set('Content-Type', 'application/json; charset=utf-8');
    return this.httpClient.delete(`/auth/users/${email}`,{headers: reqHeaders});
  }
  isAdmin() {
    const reqHeaders = new HttpHeaders();
    reqHeaders.set('Content-Type', 'application/json; charset=utf-8');
    return this.httpClient.get<boolean>("/auth/isadmin");
  }

  isValid(){
    const env = utils.get_environment();
    return localStorage.getItem(`${env}_auth_token`) != null;
  }

  getToken() {
    const env = utils.get_environment();
    return localStorage.getItem(`${env}_auth_token`);
  }
  
  logout() {
    const env = utils.get_environment();
    localStorage.removeItem(`${env}_auth_token`);
  }
}
