import { Injectable } from '@angular/core';
import { JSONLIB } from '../libraries/jsoneditor';
import { YAMLLIB } from '../libraries/yaml';
import { EventsService } from '../services/events.service';
import { timeout } from 'rxjs/operators';

import { Network } from 'vis-network';

import { Globals } from '../global';


import * as $ from "jquery";
import * as bsLib from "bootstrap-select";
import { ActivatedRoute, Router } from '@angular/router';




@Injectable({
  providedIn: 'root'
})
export class GeneralService {
  public YAML;
  monthHash = {
    0: "Jan",
    1: "Feb",
    2: "Mar",
    3: "Apr",
    4: "May",
    5: "June",
    6: "July",
    7: "Aug",
    8: "Sep",
    9: "Oct",
    10: "Nov",
    11: "Dec"
  }



  constructor(private globals: Globals, 
    private eventsService: EventsService,
    private route: ActivatedRoute,
    private router: Router,) {

    const yamlLib = YAMLLIB;
    const jsonLib = JSONLIB;

    const bootstrapLib = bsLib;


    this.YAML = window['YAML']


  }

  getApps(envs) {
    var apps_map = {};
    for (var i in envs) {
      var env = envs[i];
      var app_name = env['service'];
      if (app_name in apps_map) {
        apps_map[app_name]['envs'].push(env);
      } else {
        apps_map[app_name] = { 'name': app_name, 'envs': [env] };
      }
    }
    var ordered_map = {};
    Object.keys(apps_map).sort().forEach(key => {
      ordered_map[key] = apps_map[key];
    });
    
    return this.toArray(ordered_map);
  }

  toArray(obj) {
    var res = [];
    for (var k in obj) {
      res.push(obj[k]);
    }
    return res;
  }

  timeSince(timeStampString) {
    if (timeStampString == null) return 'Unknown';
    var timeStamp = new Date(timeStampString),
      now = new Date(),
      secondsPast = (now.getTime() - timeStamp.getTime()) / 1000;
    if (secondsPast < 60) {
      return Math.floor(secondsPast) + 's ago';
    }
    if (secondsPast < 60 * 60) {
      return Math.floor(secondsPast / 60) + 'm ago';
    }
    if (secondsPast <= 60 * 60 * 24) {
      return Math.floor(secondsPast / (60 * 60)) + 'h ago';
    }
    if (secondsPast <= 60 * 60 * 24 * 30) {
      return Math.floor(secondsPast / (60 * 60 * 24)) + 'd ago';
    }
    if (secondsPast > 60 * 60 * 24 * 30) {
      var day = timeStamp.getUTCDate();
      // console.log(timeStamp.toDateString().match(/ [a-zA-Z]*/));
      // console.log(timeStamp.toUTCString().match(/ [a-zA-Z]*/));
      //console.log(timeStamp.getUTCMonth());
      var month = this.monthHash[timeStamp.getUTCMonth()];
      var year = timeStamp.getUTCFullYear() == now.getUTCFullYear() ? "" : " " + timeStamp.getUTCFullYear();
      return month + " " + day + year;
    }
  }


  //function to get date in format YYYY-MM-DD hh:mm:ss
  getDateInUTC() {
    var date = new Date();
    var aaaa = date.getUTCFullYear();
    var ggNum = date.getUTCDate();
    var mmNum = (date.getUTCMonth() + 1);
    var gg: string = String(ggNum);
    var mm: string = String(mmNum);

    if (ggNum < 10)
      gg = "0" + gg;

    if (mmNum < 10)
      mm = "0" + mm;

    var cur_day = aaaa + "-" + mm + "-" + gg;

    var hoursNum = date.getUTCHours()
    var hoursString = String(hoursNum);
    var minutesNum = date.getUTCMinutes()
    var minutesString = String(minutesNum);
    var secondsNum = date.getUTCSeconds();
    var secondsString = String(secondsNum);

    if (hoursNum < 10)
      hoursString = "0" + hoursString;

    if (minutesNum < 10)
      minutesString = "0" + minutesString;

    if (secondsNum < 10)
      secondsString = "0" + secondsString;

    return cur_day + " " + hoursString + ":" + minutesString + ":" + secondsString;
  }

  create_form_config_builder(element_id, schema, startval, title) {


    if (title === undefined) {
      var title2 = ' ';
    }
    else {
      title2 = title;
    }


    window['JSONEditor'].defaults.editors.object.collapsed = true;
    window['JSONEditor'].defaults.editors.array.options.collapsed = false;
    schema['title'] = title2;

    // create editor


    var builder_form = new window['JSONEditor'](document.getElementById(element_id),
      {
        theme: 'bootstrap3',
        //iconlib: 'fontawesome4',
        iconlib: 'bootstrap3',
        collapsed: false,
        startval: startval,
        schema: schema,
        disable_edit_json: true,
        display_required_only: true,
        show_errors: "always",
        disable_array_delete_all_rows: true,
        disable_array_delete_last_row: true
      }
    );


    return builder_form;
  }

  create_form_environments(element_id, schema, startval, title) {

    if (title === undefined) {
      var title2 = ' ';
    }
    else {
      title2 = title;
    }



    window['JSONEditor'].defaults.editors.array.options.collapsed = true;
    // window['JSONEditor'].defaults.options.disable_edit_json = true;
    // window['JSONEditor'].defaults.options.disable_array_delete = true;
    // window['JSONEditor'].defaults.options.disable_array_add = true;
    // window['JSONEditor'].defaults.options.disable_array_reorder = true;
    // window['JSONEditor'].defaults.editors.object.options.disable_properties = true;

    var editor_schema = schema;
    editor_schema['title'] = ' ';

    // create editor
    var env_config = new window['JSONEditor'](document.getElementById(element_id),
      {
        theme: 'bootstrap3',
        //iconlib: 'fontawesome4',
        iconlib: 'bootstrap3',
        collapsed: false,
        startval: startval,
        schema: editor_schema,
        disable_edit_json: true,
        disable_array_delete: true,
        disable_array_add: true,
        disable_array_reorder: true,
        disable_properties: true
      }
    );

    return env_config;
  }

  shrink_pTag(size) {
    $('p').css('font-size', size);
  }

  set_field_help(show) {

    var help_fields = $('.json-form p');
    var hidden_class = 'hidden';
    if (show) help_fields.removeClass(hidden_class);
    else help_fields.addClass(hidden_class);
  }

  saveTextAsFile(data, filename) {

    if (!data) {
      console.error('Console.save: No data')
      return;
    }

    if (!filename) filename = 'console.json'

    var blob = new Blob([data], { type: 'text/plain' })
    //e    = document.createEvent('MouseEvents'),
    var e = new MouseEvent('click');
    var a = document.createElement('a')
    if (window.navigator && window.navigator.msSaveOrOpenBlob) {
      window.navigator.msSaveOrOpenBlob(blob, filename);
    }
    else {
      //var e = document.createEvent('MouseEvents'),
      var e = new MouseEvent('click', {
        bubbles: true,
        cancelable: false
      });
      var a = document.createElement('a');

      a.download = filename;
      a.href = window.URL.createObjectURL(blob);
      a.dataset.downloadurl = ['text/plain', a.download, a.href].join(':');

      a.dispatchEvent(e);
    }
  }

  initEnvToolbarList() {
    $(document).ready(() => {
      $('.env-select').selectpicker();
    });
  }

  initEnvToolbarTimeList() {
    $(document).ready(() => {
      $('.time-select').selectpicker();
    });

  }

  returnJQuery() {
    return $;
  }

  updateEnvlist() {
    $('.env-select').selectpicker('refresh');
  }

  setSelectedEnv(tempEnvString: string) {
    $('.env-select').selectpicker('val', tempEnvString);
  }

  createNetworkGraph(status) {

    if (!status) return;

    var images_dir = './assets/images/graph-icons'

    var env_graph_options = {
      interaction: {
        zoomView: false,
        //navigationButtons: true  // this doesn't display for some reason
      },
      layout: {
        randomSeed: 7
      }
    };


    var env_status_graph = new Network(document.getElementById('env-status-graph'), { nodes: [], edges: [] }, env_graph_options);

    // fix styling for tooltips, unfortunately can't fix via CSS directly
    var graph_el = document.getElementById('env-status-graph');
    (<HTMLElement>graph_el.children[0]).style.overflow = 'visible';
    (<HTMLElement>graph_el.children[0].children[0]).style.marginBottom = '20px';

    env_status_graph.setSize('100%', '325px');
    env_status_graph.on('click', function (properties) {
      var highlighted = 'highlighted';
      $('#env-status-detail > div > table > tbody > tr').removeClass(highlighted);
      var node = env_status_graph.getNodeAt(properties.pointer.DOM);
      if (node) {
        var id = 'inst-' + node;
        document.getElementById(id).classList.add(highlighted);
        $('#' + id).get(0).scrollIntoView();
      }
    });

    if (!('components' in status) && !('tests' in status)) return;  // exit if no data
    var data = env_status_graph.body.data;
    var center = 'center';
    var deployables = status['components'].concat(status['tests']);
    //    if (deployables.length > 1) {
    //        nodes.push({id: 'center', color: 'black', shape: 'dot', size: 0});
    //    }
    // remove nodes that no longer exist
    var exists;
    var currentNodes = data.nodes.getIds();
    for (var i = 0; i < currentNodes.length; i++) {
      exists = false;
      // check for
      for (var j = 0; j < deployables.length; j++) {
        if (currentNodes[i] == deployables[j]['name']) {
          exists = true;
          break;
        }
        for (var k = 0; k < deployables[j]['instances'].length; k++) {
          if (currentNodes[i] == deployables[j]['instances'][k]['name'] && deployables[j]['instances'][k]['state']['exists']) {
            exists = true;
            break;
          }
        }
      }
      if (!exists) {
        var connectedNodes = env_status_graph.getConnectedNodes(currentNodes[i]);
        var connectedEdges = env_status_graph.getConnectedEdges(currentNodes[i]);
        // delete connected edges
        for (var m = 0; m < connectedEdges.length; m++) data.edges.remove(connectedEdges[m]);
        // delete node
        console.log('deleting node ' + currentNodes[i]);
        data.nodes.remove(currentNodes[i]);
        // if connected node is only one and it has no more connections, delete that too
        if (connectedNodes.length == 1 && env_status_graph.getConnectedNodes(connectedNodes[0]).length == 0)
          data.nodes.remove(connectedNodes[0]);
      }
    }
    for (var i = 0; i < deployables.length; i++) {
      var d = deployables[i];
      var d_name = d['name'];
      if ('instances' in d) {
        var instances = d['instances'];
        var active_instances = 0;
        for (var j = 0; j < instances.length; j += 1) {
          var instance = d['instances'][j];
          if (instance['state']['exists']) {
            if (active_instances == 0) {
              var comp_node_defn = {
                id: d_name,
                label: d_name,
                shape: 'dot',
                size: 10,
                font: { size: 30 }
              };
              if ('urls' in d && d['urls'].length > 0) {
                comp_node_defn['shape'] = 'image';
                comp_node_defn['image'] = images_dir + '/externally-accessible.png';
                comp_node_defn['size'] = 20;
              }
              data.nodes.update(comp_node_defn);
            }
            active_instances++;
            var tooltip = '<b>' + instance['name'] + '</b>';
            if ('events' in d){
              tooltip += '<br/>Events:<br/>' + d.events;
            }
            data.nodes.update({
              id: instance['name'],
              title: tooltip,
              image: images_dir + '/' + d['kind'] + '.png',
              shape: 'image',
              shadow: {
                enabled: !instance['state']['healthy'],
                color: instance['state']['name'] == 'Pending' ? 'yellow' : 'red'
              },
              font: {
                size: 16,
                color: 'black'
              }
            });
            data.edges.update({
              id: instance['name'] + '-' + d_name,
              from: instance['name'],
              to: d_name
            });
          }
        }
      }
    }
    env_status_graph.on('initRedraw', env_status_graph.fit);
    // add event handler to open url of component when node is clicked
    env_status_graph.off('selectNode');
    env_status_graph.on('selectNode', function (params) {
      env_status_graph.unselectAll();
      var node_name = params['nodes'][0]
      for (var i = 0; i < deployables.length; i++) {
        if (deployables[i].name == node_name) {
          if ('urls' in deployables[i] && deployables[i].urls.length > 0) window.open(deployables[i].urls[0]);
          break;
        }
      }
    });
  }

  split_env_string(s) {
    var parts = s.split(' ');
    var app_string = parts[0].split('/')[0];
    var env_string = parts[0].split('/')[1];
    var cluster_string = parts.length > 1 ? parts[1].substr(1, parts[1].length - 2) : '';
    return [app_string, env_string, cluster_string];
  }

  scrollToBottom(element){
    // if (!force){
    //   var isScrolledToBottom = element.scrollHeight - element.clientHeight <= element.scrollTop + 1;
    //   if (isScrolledToBottom) {
    //     element.scrollTop = element.scrollHeight - element.clientHeight;
    //   }
    // } else {

    //   element.scrollTop = element.scrollHeight - element.clientHeight;
    // }
    element.scrollTop = element.scrollHeight - element.clientHeight;
  }
  alterQueryParameters(paramObj) {
    this.router.navigate([], {
      queryParams: paramObj,
      queryParamsHandling: 'merge',
    });
  }

}




