import { Injectable, Inject } from '@angular/core';
import { HttpInterceptor, HttpEvent, HttpClient, HttpHandler, HttpHeaders, HttpRequest } from '@angular/common/http';
import { Observable } from 'rxjs';
import { AuthService } from './auth.service';
import { environment } from '../../environments/environment';
import * as utils from './utils';

@Injectable({
  providedIn: 'root'
})

@Injectable()
export class UrlInterceptorService {


  constructor(
    @Inject('API_URL') private apiUrl: string, @Inject('AUTH_API_URL') private authApiUrl: string) {
      //console.log("creating interceptor: ", this.apiUrl);
      if(environment.production) {
        this.apiUrl  = '../../api/80/';
        this.authApiUrl = '../../auth-api/8081/';
      }
    }


  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {

    const urlParts = request.url.split('/')
    let newUrl = urlParts[1] === 'auth' ? `${this.authApiUrl}${request.url}` : `${this.apiUrl}${request.url}`;
    const apiReq = request.clone({ url: newUrl });

    return next.handle(apiReq);
  }
}
