import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { User } from '../models/user';
import { AuthService } from '../services/auth.service';

@Component({
  selector: 'app-user-create',
  templateUrl: './user-create.component.html',
  styleUrls: ['./user-create.component.css']
})
export class UserCreateComponent implements OnInit {

  userCreateForm : FormGroup;
  public user : User;
  public action : string; 

  constructor(public dialogRef: MatDialogRef<UserCreateComponent>,
    private formBuilder: FormBuilder, 
    private authService: AuthService,
    @Inject(MAT_DIALOG_DATA) public data ) { 
      if(this.data){
        this.user = this.data['user'];
      } 
      this.action = this.user? 'Edit' : 'Create';
    }

  ngOnInit(): void {
    this.userCreateForm = this.formBuilder.group({
      email : [this.user ? this.user.email : '', [Validators.required]],
      password : [null],
      roles: [this.user? this.user.roles : 'dev'],
      githubUsername : [this.user? this.user.github_username: ''],
      githubOrg:[this.user? this.user.github_org: '']
    })
  }

  submit() {
    if(!this.userCreateForm.valid) {
      return;
    }
    const params = {
      'email': this.userCreateForm.value['email'],
      'password': this.userCreateForm.value['password'],
      'github_username': this.userCreateForm.value['githubUsername'],
      'roles': this.userCreateForm.value['roles'],
      'github_org': this.userCreateForm.value['githubOrg'],
    }

    if(this.user) {
      this.authService.editUser(this.user.email, params).subscribe(data => {
        this.dialogRef.close();
      })
    }else {
      this.authService.createUser(params).subscribe(data => {
        this.dialogRef.close();
      });
    }

    
  }

}
